// Core dependencies
import { BrowserModule } from '@angular/platform-browser';
import { NgModule, APP_INITIALIZER } from '@angular/core';
import { BrowserAnimationsModule } from '@angular/platform-browser/animations';
import { HttpClientModule } from '@angular/common/http';
import { FormsModule } from '@angular/forms';

// Vendor dependencies
import { AlertModule } from 'ngx-bootstrap/alert';
import { ModalModule } from 'ngx-bootstrap/modal';
import { DateValueAccessorModule } from 'angular-date-value-accessor';
import { ToasterModule, ToasterService } from 'angular2-toaster';

// App dependencies

import { AppRoutingModule } from './app-routing.module';
import { AppComponent } from './app.component';

import { 
  AppConfigService, 
  AppConfigServiceFactory, 
  ActivityService, 
  DataService,
  EntryService,
  IntensityService,
  MessageService,
  ModelStateService
} from './services';

import { EntryListComponent } from './entry-list/entry-list.component';
import { EntryAddComponent } from './entry-add/entry-add.component';
import { EntryEditComponent } from './entry-edit/entry-edit.component';
import { EntryDeleteComponent } from './entry-delete/entry-delete.component';
import { PageNotFoundComponent } from './page-not-found/page-not-found.component';
import { LogoComponent } from './logo/logo.component';
import { EntryFormComponent } from './entry-form/entry-form.component';
import { EntryRowComponent } from './entry-row/entry-row.component';
import { ServerErrorComponent } from './server-error/server-error.component';
import { NavbarComponent } from './navbar/navbar.component';

@NgModule({
  entryComponents: [
    EntryDeleteComponent,
    ServerErrorComponent
  ],
  declarations: [
    AppComponent,
    EntryListComponent,
    EntryAddComponent,
    EntryEditComponent,
    EntryDeleteComponent,
    PageNotFoundComponent,
    LogoComponent,
    EntryFormComponent,
    EntryRowComponent,
    ServerErrorComponent,
    NavbarComponent
  ],
  imports: [
    BrowserModule,
    BrowserAnimationsModule, 
    HttpClientModule,
    FormsModule,
    ToasterModule,
    AppRoutingModule,
    DateValueAccessorModule,
    AlertModule.forRoot(),
    ModalModule.forRoot()
  ],
  providers: [
    AppConfigService,
    ActivityService,
    DataService,
    EntryService,
    IntensityService,
    MessageService,
    ModelStateService,
    { provide: APP_INITIALIZER, useFactory: AppConfigServiceFactory, deps: [AppConfigService], multi: true }
  ],
  bootstrap: [AppComponent]
})
export class AppModule { }
