﻿import { Component, ViewChild } from '@angular/core';
import { HttpErrorResponse } from '@angular/common/http';
import { Router } from '@angular/router';
import { Entry } from '../models';
import { 
  AppConfigService, 
  EntryService, 
  MessageService, 
  ModelStateService 
} from '../services';
import { EntryFormComponent } from '../entry-form/entry-form.component';
import { ServerErrorComponent } from '../server-error/server-error.component';
import { BsModalService } from 'ngx-bootstrap/modal';
import { BsModalRef } from 'ngx-bootstrap/modal/modal-options.class';

@Component({
  selector: 'app-entry-add',
  templateUrl: './entry-add.component.html',
  styleUrls: ['./entry-add.component.css']
})
export class EntryAddComponent {
  title: string = "Add Entry";
  entry: Entry = new Entry();
  serverErrors: string[] = [];
  @ViewChild('entryForm') entryForm: EntryFormComponent;

  constructor(
    private appConfigService: AppConfigService,
    private entryService: EntryService, 
    private messageService: MessageService,
    private modelStateService: ModelStateService,
    private router: Router,
    private modalService: BsModalService) { }

  submit(entry: Entry) {
    if (this.appConfigService.appConfig.useInMemoryData) {
      entry.updateDateToLocalTime();
      entry.updateActivity(this.entryForm.activities);
      entry.updateIntensity(this.entryForm.intensities);
    }

    this.entryService.add(entry)
      .subscribe(data => {
        this.messageService.displayMessage('Your entry was successfully added!');
        this.router.navigate(['entries']);
      }, (err: HttpErrorResponse) => {
        if (err.error && err.error.modelState) {
          this.modelStateService.displayModelStateErrorMessages(
            err.error.modelState, 
            this.entryForm.currentForm, 
            this.entryForm.serverErrors);
        } else {
          const modalRef = this.modalService.show(ServerErrorComponent);
          const serverErrorComponent = <ServerErrorComponent>modalRef.content;
          serverErrorComponent.error = err;
        }
      });
  }
}
