﻿import { Component, OnInit, ViewChild } from '@angular/core';
import { HttpErrorResponse } from '@angular/common/http';
import { Router, ActivatedRoute, ParamMap } from '@angular/router';
import { Entry } from '../models';
import { 
  AppConfigService, 
  EntryService, 
  MessageService, 
  ModelStateService 
} from '../services';
import { EntryFormComponent } from '../entry-form/entry-form.component';
import { ServerErrorComponent } from '../server-error/server-error.component';
import { BsModalService } from 'ngx-bootstrap/modal';
import { BsModalRef } from 'ngx-bootstrap/modal/modal-options.class';
import 'rxjs/add/operator/switchMap';

@Component({
  selector: 'app-entry-edit',
  templateUrl: './entry-edit.component.html',
  styleUrls: ['./entry-edit.component.css']
})
export class EntryEditComponent implements OnInit {
  title: string = "Edit Entry";
  entry: Entry = new Entry();
  serverErrors: string[] = [];
  @ViewChild('entryForm') entryForm: EntryFormComponent;

  constructor(
    private appConfigService: AppConfigService,
    private entryService: EntryService, 
    private messageService: MessageService,
    private modelStateService: ModelStateService,
    private route: ActivatedRoute,
    private router: Router,
    private modalService: BsModalService) { }

  ngOnInit() {
    this.route.paramMap
      .switchMap(params =>
        this.entryService.get(parseInt(params.get('id'), 10)))
      .subscribe(entry => {
        entry.updateDateToUtcTime();
        this.entry = entry;
      });
  }
  
  submit(entry: Entry) {
    if (this.appConfigService.appConfig.useInMemoryData) {
      entry.updateDateToLocalTime();
      entry.updateActivity(this.entryForm.activities);
      entry.updateIntensity(this.entryForm.intensities);
    }

    this.entryService.update(entry)
      .subscribe(data => {
        this.messageService.displayMessage('Your entry was successfully updated!');
        this.router.navigate(['entries']);
      }, (err: HttpErrorResponse) => {
        if (err.error && err.error.modelState) {
          this.modelStateService.displayModelStateErrorMessages(
            err.error.modelState,
            this.entryForm.currentForm, 
            this.entryForm.serverErrors);
        } else {
          const modalRef = this.modalService.show(ServerErrorComponent);
          const serverErrorComponent = <ServerErrorComponent>modalRef.content;
          serverErrorComponent.error = err;
        }
      });
  }
}
