import { Component, OnInit, ViewChild, Input, Output, EventEmitter } from '@angular/core';
import { NgForm } from '@angular/forms';
import { Activity, Entry, Intensity, ModelState } from '../models';
import { ActivityService, IntensityService } from '../services';

@Component({
  selector: 'app-entry-form',
  templateUrl: './entry-form.component.html',
  styleUrls: ['./entry-form.component.css']
})
export class EntryFormComponent implements OnInit {
  @Input() entry: Entry = new Entry();
  activities: Activity[];
  intensities: Intensity[];
  @Input() serverErrors: string[] = [];
  @ViewChild('entryForm') currentForm: NgForm;
  @Output() onSubmit: EventEmitter<Entry> = new EventEmitter<Entry>();

  constructor(
    private activityService: ActivityService, 
    private intensityService: IntensityService) { }

  ngOnInit() {
    this.activityService.getList().subscribe(data => {
      this.activities = data;
    });

    this.intensityService.getList().subscribe(data => {
      this.intensities = data;
    });
  }

  submit() {
    // Reset the server errors array of messages.
    this.serverErrors = [];

    this.onSubmit.emit(this.entry);
  }
}
