﻿import { Component, OnInit, ViewChild, Input, Output, EventEmitter } from '@angular/core';
import { HttpErrorResponse } from '@angular/common/http';
import { NgForm } from '@angular/forms';
import { Activity, Entry, Intensity } from '../models';
import { 
  ActivityService, 
  EntryService, 
  IntensityService, 
  ModelStateService 
} from '../services';
import { ServerErrorComponent } from '../server-error/server-error.component';
import { BsModalService } from 'ngx-bootstrap/modal';
import { BsModalRef } from 'ngx-bootstrap/modal/modal-options.class';
import { Observable } from 'rxjs/Observable';
import 'rxjs/add/observable/forkJoin';

@Component({
  selector: 'tr[entry-row]',
  templateUrl: './entry-row.component.html',
  styleUrls: ['./entry-row.component.css']
})
export class EntryRowComponent implements OnInit {
  @Input() entry: Entry;
  entryEdit: Entry;
  @Output() onEditCompleted: EventEmitter<Entry> = new EventEmitter<Entry>();
  @Output() onConfirmDeletion: EventEmitter<Entry> = new EventEmitter<Entry>();
  isEditing: boolean = false;
  activities: Activity[];
  intensities: Intensity[];
  serverErrors: string[] = [];
  @ViewChild('entryForm') currentForm: NgForm;

  constructor(
    private activityService: ActivityService, 
    private entryService: EntryService,
    private intensityService: IntensityService,
    private modelStateService: ModelStateService,
    private modalService: BsModalService) { }

  ngOnInit() {
  }

  edit() {
    Observable.forkJoin(
      this.activityService.getList(),
      this.intensityService.getList()
    ).subscribe(data => {
      this.activities = data[0];
      this.intensities = data[1];
      const entryEdit = new Entry(this.entry);
      entryEdit.updateDateToUtcTime();
      this.entryEdit = entryEdit;
      this.isEditing = true;
    });
  }

  submit() {
    const entryEdit = this.entryEdit;

    this.entryService.update(entryEdit)
      .subscribe(data => {
        this.isEditing = false;

        entryEdit.updateDateToLocalTime();
        entryEdit.updateActivity(this.activities);
        entryEdit.updateIntensity(this.intensities);

        this.entry = entryEdit;
        this.entryEdit = null;

        this.onEditCompleted.emit(entryEdit);
      }, (err: HttpErrorResponse) => {
        if (err.error && err.error.modelState) {
          this.modelStateService.displayModelStateErrorMessages(
            err.error.modelState,
            this.currentForm, 
            this.serverErrors);
        } else {
          const modalRef = this.modalService.show(ServerErrorComponent);
          const serverErrorComponent = <ServerErrorComponent>modalRef.content;
          serverErrorComponent.error = err;
        }
      });
  }

  cancel() {
    this.isEditing = false;
    this.entryEdit = null;
  }

  confirmDeletion(entry: Entry) {
    this.onConfirmDeletion.emit(entry);
  }
}
