﻿
import { Activity } from './activity';
import { Intensity } from './intensity';

export interface EntryInterface {
  id: number;
  date: Date;
  activityId: number;
  activity: Activity;
  duration: number;
  intensity: number;
  exclude?: boolean;
  notes?: string;
  intensityName: string;
}

export class Entry {
  id: number;
  activityId: number;
  activity: Activity = new Activity();
  duration: number;
  intensity: number;
  exclude: boolean;
  notes: string;
  intensityName: string;

  constructor(entry?: EntryInterface) {
    if (entry) {
      this.id = entry.id;
      this.date = new Date(entry.date);
      this.activityId = entry.activityId;
      if (entry.activity) {
        this.activity = new Activity(entry.activity.id, entry.activity.name);
      }
      this.duration = entry.duration;
      this.intensity = entry.intensity;
      this.exclude = entry.exclude;
      this.notes = entry.notes;
      this.intensityName = entry.intensityName;
    } else {
      this.date = this.getUtcToday();
      this.intensity = 1;
    }
  }

  private _dateAsDate: Date;

  get date(): any {
    return this._dateAsDate;
  }

  set date(value: any) {
    if (value instanceof Date) {
      this._dateAsDate = value;
    } else {
      this._dateAsDate = new Date(value);
    }
  }

  updateDateToUtcTime() {
    this.date.setMinutes(
      this.date.getMinutes() - this.date.getTimezoneOffset());    
  }

  updateDateToLocalTime() {
    this.date.setMinutes(
      this.date.getMinutes() + this.date.getTimezoneOffset());
  }

  updateActivity(activities: Activity[]) {
    const activity = activities.find(a => a.id === this.activityId);
    this.activity = new Activity(activity.id, activity.name);
  }

  updateIntensity(intensities: Intensity[]) {
    this.intensityName = intensities.find(i => i.id === this.intensity).name;
  }

  private getUtcToday(): Date {
    const date = new Date();
    const dateNoTime = new Date(date.toDateString());
    dateNoTime.setMinutes(
      dateNoTime.getMinutes() - dateNoTime.getTimezoneOffset());
    return dateNoTime;
  }

  static sortEntries(entries: Entry[]) {
    entries.sort((entry1, entry2) => {
      // We want a descending sort on "date"
      // so return "1" if the entry1 date is less than the entry2 date
      // and vice versa.
      if (entry1.date < entry2.date) {
        return 1;
      } else if (entry1.date > entry2.date) {
        return -1;
      } else {
        if (entry1.id < entry2.id) {
          return 1;
        } else if (entry1.id > entry2.id) {
          return -1;
        } else {
          return 0;
        }
      }
    })
  }
  
  toJSON() {
    return {
      "id": this.id,
      "date": this.date,
      "activityId": this.activityId,
      "duration": this.duration,
      "intensity": this.intensity,
      "exclude": this.exclude,
      "notes": this.notes
    };
  }
}
