import { HttpClient, HttpErrorResponse } from '@angular/common/http';
import { Observable } from 'rxjs/Observable';
import 'rxjs/add/operator/catch';

export abstract class BaseService {
  constructor(private http: HttpClient, private apiBaseUrl: string) { }

  protected getRequest<TItemType>(path: string) {
    const request = this.http.get<TItemType>(this.apiBaseUrl + path);

    request.catch((e, caught) => {
      this.handleError(e);
      return Observable.throw(e);
    });

    return request;
  }

  protected postRequest(path: string, model: any) {
    const request = this.http.post(this.apiBaseUrl + path, model);

    request.catch((e, caught) => {
      this.handleError(e);
      return Observable.throw(e);
    });

    return request;
  }

  protected putRequest(path: string, model: any) {
    const request = this.http.put(this.apiBaseUrl + path, model);

    request.catch((e, caught) => {
      this.handleError(e);
      return Observable.throw(e);
    });

    return request;
  }

  protected deleteRequest(path: string) {
    const request = this.http.delete(this.apiBaseUrl + path);

    request.catch((e, caught) => {
      this.handleError(e);
      return Observable.throw(e);
    });

    return request;
  }

  private handleError(err: any) {
    if (err instanceof HttpErrorResponse) {
      if (err.error instanceof Error) {
        // A client-side or network error occurred. Handle it accordingly.
        console.error(`An error occurred: ${err.error.message}`);
      } else {
        // The backend returned an unsuccessful response code.
        // The response body may contain clues as to what went wrong,
        console.error(`Backend returned code ${err.status}, body was: ${err.error}`);
      }
    } else {
      console.error(`An error occurred: ${err}`);
    }
  }
}
