import { Injectable } from '@angular/core';
import { Activity, Entry, Intensity } from '../models';
import { AppConfigService } from '../services';

@Injectable()
export class DataService {
  activities: Activity[];
  entries: Entry[];
  intensities: Intensity[];

  constructor(private appConfigService: AppConfigService) {
    if (!appConfigService.appConfig.useInMemoryData) {
      return;
    }

    const activityBasketball = new Activity(1, 'Basketball');
    const activityBiking = new Activity(2, 'Biking');
    const activityHiking = new Activity(3, 'Hiking');
    const activityKayaking = new Activity(4, 'Kayaking');
    const activityPokemonGo = new Activity(5, 'Pokemon Go');
    const activityRunning = new Activity(6, 'Running');
    const activitySkiing = new Activity(7, 'Skiing');
    const activitySwimming = new Activity(8, 'Swimming');
    const activityWalking = new Activity(9, 'Walking');
    const activityWeightLifting = new Activity(10, 'Weight Lifting');

    this.activities = [
      activityBasketball,
      activityBiking, 
      activityHiking,
      activityKayaking,
      activityPokemonGo,
      activityRunning,
      activitySkiing,
      activitySwimming,
      activityWalking,
      activityWeightLifting
    ];

    const intensityLow = new Intensity(1, 'Low');
    const intensityMedium = new Intensity(2, 'Medium');
    const intensityHigh = new Intensity(3, 'High');

    this.intensities = [
      intensityLow,
      intensityMedium,
      intensityHigh
    ];

    const entry1 = new Entry({
      id: 1,
      date: new Date(2017, 6, 8),
      activityId: activityBiking.id,
      activity: activityBiking,
      duration: 10,
      intensity: 1,
      intensityName: 'Low'
    });

    const entry2 = new Entry({
      id: 2,
      date: new Date(2017, 6, 9),
      activityId: activityBiking.id,
      activity: activityBiking,
      duration: 12.2,
      intensity: 1,
      intensityName: 'Low'
    });

    const entry3 = new Entry({
      id: 3,
      date: new Date(2017, 6, 10),
      activityId: activityBiking.id,
      activity: activityBiking,
      duration: 123,
      intensity: 1,
      intensityName: 'Low'
    });

    const entry4 = new Entry({
      id: 4,
      date: new Date(2017, 6, 12),
      activityId: activityBiking.id,
      activity: activityBiking,
      duration: 10,
      intensity: 1,
      intensityName: 'Low'
    });

    const entry5 = new Entry({
      id: 5,
      date: new Date(2017, 6, 13),
      activityId: activityWalking.id,
      activity: activityWalking,
      duration: 32.2,
      intensity: 1,
      intensityName: 'Low'
    });

    const entry6 = new Entry({
      id: 6,
      date: new Date(2017, 6, 13),
      activityId: activityBiking.id,
      activity: activityBiking,
      duration: 13.3,
      intensity: 1,
      intensityName: 'Low'
    });

    const entry7 = new Entry({
      id: 7,
      date: new Date(2017, 6, 14),
      activityId: activityBiking.id,
      activity: activityBiking,
      duration: 10,
      intensity: 1,
      intensityName: 'Low'
    });

    const entry8 = new Entry({
      id: 8,
      date: new Date(2017, 6, 15),
      activityId: activityWalking.id,
      activity: activityWalking,
      duration: 28.6,
      intensity: 1,
      intensityName: 'Low'
    });

    const entry9 = new Entry({
      id: 9,
      date: new Date(2017, 6, 16),
      activityId: activityBiking.id,
      activity: activityBiking,
      duration: 12.7,
      intensity: 1,
      intensityName: 'Low'
    });

    const entry10 = new Entry({
      id: 10,
      date: new Date(2017, 6, 16),
      activityId: activityPokemonGo.id,
      activity: activityPokemonGo,
      duration: 23.4,
      intensity: 1,
      intensityName: 'Low'
    });

    this.entries = [
      entry1,
      entry2,
      entry3,
      entry4,
      entry5,
      entry6,
      entry7,
      entry8,
      entry9,
      entry10
    ];

    Entry.sortEntries(this.entries);
  }

  addEntry(entry: Entry) {
    entry.id = this.entries.length + 1;
    this.entries.push(entry);
    Entry.sortEntries(this.entries);
  }

  updateEntry(entry: Entry) {
    const entryIndex = this.entries
      .findIndex(e => e.id === entry.id);
    this.entries[entryIndex] = entry;
    Entry.sortEntries(this.entries);
  }

  deleteEntry(id: number) {
    const entryIndex = this.entries
      .findIndex(e => e.id === id);
    this.entries.splice(entryIndex, 1);
  }
}
