import { Injectable } from '@angular/core';
import { HttpClient } from '@angular/common/http';
import { Entry } from '../models';
import { AppConfigService, DataService } from './';
import { BaseService } from './base.service';
import { environment } from '../../environments/environment';
import { Observable } from 'rxjs/Observable';
import 'rxjs/add/observable/of';
import 'rxjs/add/operator/map';

@Injectable()
export class EntryService extends BaseService {
  constructor(
      http: HttpClient, 
      private appConfigService: AppConfigService, 
      private dataService: DataService) {
    super(http, environment.apiBaseUrl);
  }

  getList(): Observable<Entry[]> {
    if (this.appConfigService.appConfig.useInMemoryData) {
      return Observable.of(this.dataService.entries);
    }

    let request = super.getRequest<Entry[]>('entries');

    // Map each object to an instance of the Entry class.
    return request.map(entries => {
      entries.forEach(entry => {
        entry = new Entry(entry);
      });

      return entries;
    });
  }

  get(id: number) {
    if (this.appConfigService.appConfig.useInMemoryData) {
      const entry = this.dataService.entries.find(e => e.id === id);
      return Observable.of(new Entry(entry));
    }

    const request = super.getRequest<Entry>(`entries/${id}`);

    // Map the object to an instance of the Entry class.
    return request.map(entry => {
      return new Entry(entry);
    });
  }

  add(entry: Entry) {
    if (this.appConfigService.appConfig.useInMemoryData) {
      this.dataService.addEntry(entry);
      return Observable.of({});
    }

    return super.postRequest('entries', entry);
  }

  update(entry: Entry) {
    if (this.appConfigService.appConfig.useInMemoryData) {
      this.dataService.updateEntry(entry);
      return Observable.of({});
    }

    return super.putRequest(`entries/${entry.id}`, entry);    
  }

  delete(entry: Entry) {
    if (this.appConfigService.appConfig.useInMemoryData) {
      this.dataService.deleteEntry(entry.id);
      return Observable.of({});
    }

    return super.deleteRequest(`entries/${entry.id}`);
  }
}
