import { Injectable } from '@angular/core';
import { NgForm } from '@angular/forms';
import { ModelState } from '../models';

@Injectable()
export class ModelStateService {

  constructor() { }

  displayModelStateErrorMessages(modelState: ModelState, 
      currentForm: NgForm, serverErrors: string[]) {
    for (const modelStateFieldName in modelState) {
      // Get the client-side field name from the server-side 
      // model state field name.
      const fieldName = this.getFieldNameFromModelStateKey(modelStateFieldName);

      // HACK To set the field to an invalid state 
      // call the form control's setErrors method and 
      // pass in an errors object with a single "serverErrors" property.
      currentForm.form.controls[fieldName].setErrors({
        serverErrors: true
      });

      // Push all of the error messages 
      // onto the serverErrors array of messages.
      serverErrors.push(...modelState[modelStateFieldName]);
    }
  }

  private getFieldNameFromModelStateKey(key: string): string {
    let fieldName = key;

    // If the field name contains the entity name 
    // in the form "{entity name}.{field name}" strip it off.
    if (fieldName.indexOf('.') !== -1) {
      fieldName = fieldName.slice(fieldName.indexOf('.') + 1);
    }

    // Ensure that the field's first character is lower cased.
    fieldName = fieldName.slice(0, 1).toLowerCase() + fieldName.slice(1);

    return fieldName;
  }
}
